# JumpList Parser
#
# Copyright (C) 2014 Plainbit
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details. 
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

"""
@author       : MaJ3stY(Lee Jun Hyeong)
@license      : GNU General Public License 2.0 or later
@contact      : junhyeong@plainbit.co.kr
@organization : http://plainbit.co.kr, http://maj3sty.tistory.com
"""

__author__ = 'MaJ3stY'
# -*- coding:  euc-kr -*-

import OleFileIO_PL
import struct
import binascii
import time
from optparse import OptionParser
import sys
import os
import sqlite3

# See http://www.forensicswiki.org/wiki/List_of_Jump_List_IDs
AppIDList = {\
    '65009083bfa6a094':'(app launched via XPMode)', \
    '89b0d939f117f75c':'Adobe Acrobat 9 Pro Extended x86', \
    '26717493b25aa6e1':'Adobe Dreamweaver CS5 x86', \
    'e2a593822e01aed3':'Adobe Flash CS5 x86', \
    'c765823d986857ba':'Adobe Illustrator CS5 x86', \
    '84f066768a22cc4f':'Adobe Photoshop CS5 x64', \
    '44a398496acc926d':'Adobe Premiere Pro CS5 x64', \
    '23646679aaccfae0':'Adobe Reader 9 x86 or x64', \
    'd5c3931caad5f793':'Adobe Soundbooth CS5 x86', \
    '7e4dca80246863e3':'Control Panel (?)', \
    'bc03160ee1a59fc1':'Foxit PDF Reader 5.4.5', \
    '28c8b86deab549a1':'Internet Explorer 8 / 9 / 10 x86', \
    '5da8f997fd5f9428':'Internet Explorer x64', \
    '271e609288e1210a':'Microsoft Office Access 2010 x86', \
    'cdf30b95c55fd785':'Microsoft Office Excel 2007', \
    '9839aec31243a928':'Microsoft Office Excel 2010 x86', \
    '6e855c85de07bc6a':'Microsoft Office Excel 2010 x64', \
    'b8c29862d9f95832':'Microsoft Office InfoPath 2010 x86', \
    'd64d36b238c843a3':'Microsoft Office InfoPath 2010 x86', \
    '3094cdb43bf5e9c2':'Microsoft Office OneNote 2010 x86', \
    'd38adec6953449ba':'Microsoft Office OneNote 2010 x64', \
    'be71009ff8bb02a2':'Microsoft Office Outlook x86', \
    '5d6f13ed567aa2da':'Microsoft Office Outlook 2010 x64', \
    'f5ac5390b9115fdb':'Microsoft Office PowerPoint 2007', \
    '9c7cc110ff56d1bd':'Microsoft Office PowerPoint 2010 x86', \
    '5f6e7bc0fb699772':'Microsoft Office PowerPoint 2010 x64', \
    'a8c43ef36da523b1':'Microsoft Office Word 2003 Pinned and Recent.', \
    'adecfb853d77462a':'Microsoft Office Word 2007 Pinned and Recent.', \
    'a7bd71699cd38d1c':'Microsoft Office Word 2010 x86', \
    '44a3621b32122d64':'Microsoft Office Word 2010 x64', \
    '12dc1ea8e34b5a6':'Microsoft Paint 6.1', \
    'e70d383b15687e37':'Notepad++ 5.6.8 x86', \
    '918e0ecb43d17e23':'Notepad x86', \
    '9b9cdc69c1c24e2b':'Notepad x64', \
    'c7a4093872176c74':'Paint Shop Pro Pinned and Recent.', \
    'c71ef2c372d322d7':'PGP Desktop 10', \
    '431a5b43435cc60b':'Python (.pyc)', \
    '500b8c1d5302fc9c':'Python (.pyw)', \
    '1bc392b8e104a00e':'Remote Desktop', \
    '315e29a36e961336':'Roboform 7.8', \
    '17d3eb086439f0d7':'TrueCrypt 7.0a', \
    '50620fe75ee0093':'VMware Player 3.1.4', \
    '8eafbd04ec8631ce':'VMware Workstation 9 x64', \
    '6728dd69a3088f97':'Windows Command Processor - cmd.exe x64', \
    '1b4dd67f29cb1962':'Windows Explorer Pinned and Recent.', \
    'd7528034b5bd6f28':'Windows Live Mail Pinned and Recent.', \
    'b91050d8b077a4e8':'Windows Media Center x64', \
    '43578521d78096c6':'Windows Media Player Classic Home Cinema 1.3 x86', \
    '954ea5f70258b502':'Windows Script Host - wscript.exe x86', \
    '9f5c7755804b850a':'Windows Script Host - wscript.exe x64', \
    'b0459de4674aab56':'Windows Virtual PC - vmwindow.exe x86/64 or (.vmcx)', \
    '469e4a7982cea4d4':'Windows Wordpad', \
    'e36bfc8972e5ab1d':'XPS Viewer', \
    '2b53c4ddf69195fc':'Zune x64', \
    'f0468ce1ae57883d':'Adobe Reader 7.1.0', \
    'c2d349a0e756411b':'Adobe Reader 8.1.2', \
    'ee462c3b81abb6f6':'Adobe Reader X 10.1.0', \
    '386a2f6aa7967f36':'EyeBrowse 2.7', \
    'e31a6a8a7506f733':'Image AXS Pro 4.1', \
    'b39c5f226977725d':'ACDSee Pro 8.1.99', \
    '59f56184c796cfd4':'ACDSee Photo Manager 10 (Build 219)', \
    '8bd5c6433ca967e9':'ACDSee Photo Manager 2009 (v11.0 Build 113)', \
    'd838aac097abece7':'ACDSee Photo Manager 12 (Build 344)', \
    'b3f13480c2785ae':'Paint 6.1 (build 7601: SP1)', \
    '7cb0735d45243070':'CDisplay 1.8.1.0', \
    '3594aab44bca414b':'Windows Photo Viewer', \
    '3edf100b207e2199':'digiKam 1.7.0 (KDE 4.4.4)', \
    '169b3be0bc43d592':'FastPictureViewer Professional 1.6 (Build 211)', \
    'e9a39dfba105ea23':'FastStone Image Viewer 4.6', \
    'ef606b196796ebb':'HP MediaSmart Photo', \
    'edc786643819316c':'HoneyView3 #5834', \
    '76689ff502a1fd9e':'Imagine Image and Animation Viewer 1.0.7', \
    '2519133d6d830f7e':'IMatch 3.6.0.113', \
    '1110d9896dceddb3':'imgSeek 0.8.5', \
    'c634153e7f5fce9c':'IrfanView 3.10 / 4.30', \
    'ea83017cdd24374d':'IrfanView Thumbnails', \
    '3917dd550d7df9a8':'Konvertor 4.06 (Build 10)', \
    '2fa14c7753239e4c':'Paint.NET 2.72 / 3.5.8.4081.24580', \
    'd33ecf70f0b74a77':'Picasa 2.2.0 (Build 28.08, 0)', \
    'b17d3d0c9ca7e29':'Picasa 3.8.0 (Build 117.43, 0)', \
    'c5c24a503b1727df':'XnView 1.98.2 Small / 1.98.2 Standard', \
    '497b42680f564128':'Zoner PhotoStudio 13 (Build 7)', \
    '5d696d521de238c3':'Chrome 9.0.597.84 / 12.0.742.100 / 13.0.785.215 / 26', \
    'cfb56c56fa0f0a54':'Mozilla 0.9.9', \
    '5c450709f7ae4396':'Firefox 1.0 / 2.0 / 3.0 x86', \
    '5df4765359170e26':'Firefox 4.0.1', \
    '1eb796d87c32eff9':'Firefox 5.0', \
    '1461132e553e2e6c':'Firefox 6.0', \
    '16ec093b8f51508f':'Opera 8.54 build 7730 / 9.64 build 10487 / 11.50 build 1074', \
    '8a1c1c7c389a5320':'Safari 3.2.3 (525.29)', \
    '1da3c90a72bf5527':'Safari 4.0.5 (531.22.7) / 5.1 (7534.50)', \
    'e0f7a40340179171':'imule 1.4.5 (rev. 749)', \
    '76f6f1bd18c19698':'aMule 2.2.6', \
    'cb5250eaef7e3213':'ApexDC++ 1.4.3.957', \
    'bfc1d76f16fa778f':'Ares (Galaxy) 1.8.4 / 1.9.8 / 2.1.0 / 2.1.7.3041', \
    'accca100973ef8dc':'Azureus 2.0.8.4', \
    'ccb36ff8a8c03b4b':'Azureus 2.5.0.4 / Vuze 3.0.5.0', \
    '558c5bd9f906860a':'BearShare Lite 5.2.5.1', \
    'e1d47cb031dafb9f':'BearShare 6.0.0.22717 / 8.1.0.70928 / 10.0.0.112380', \
    'a31ec95fdd5f350f':'BitComet 0.49 / 0.59 / 0.69 / 0.79 / 0.89 / 0.99 / 1.07 / 1.28', \
    'bcd7ba75303acbcf':'BitLord 1.1', \
    '1434d6d62d64857d':'BitLord 1.2.0-66', \
    'e73d9f534ed5618a':'BitSpirit 1.2.0.228 / 2.0 / 2.6.3.168 / 2.7.2.239 / 2.8.0.072 / 3.1.0.077 / 3.6.0.550', \
    'c9374251edb4c1a8':'BitTornado T-0.3.17', \
    '2d61cccb4338dfc8':'BitTorrent 5.0.0 / 6.0.0 / 7.2.1 (Build 25548)', \
    'ba3a45f7fd2583e1':'Blubster 3.1.1', \
    '4a7e4f6a181d3d08':'broolzShare', \
    'f001ea668c0aa916':'Cabos 0.8.2', \
    '560d789a6a42ad5a':'DC++ 0.261 / 0.698 / 0.782 (r2402.1)', \
    '4aa2a5710da3efe0':'DCSharpHub 2.0.0', \
    '2db8e25112ab4453':'Deluge 1.3.3', \
    '5b186fc4a0b40504':'Dtella 1.2.5 (Purdue network only)', \
    '2437d4d14b056114':'EiskaltDC++ 2.2.3', \
    'b3016b8da2077262':'eMule 0.50a', \
    'cbbe886eca4bfc2d':'ExoSee 1.0.0', \
    '9ad1ec169bf2da7f':'FlylinkDC++ r405 (Build 7358)', \
    '4dd48f858b1a6ba7':'Free Download Manager 3.0 (Build 852)', \
    'f214ca2dd40c59c1':'FrostWire 4.20.9', \
    '73ce3745a843c0a4':'FrostWire 5.1.4', \
    '98b0ef1c84088':'fulDC 6.78', \
    'e6ea77a1d4553872':'Gnucleus 1.8.6.0', \
    'ed49e1e6ccdba2f5':'GNUnet 0.8.1a', \
    'cc4b36fbfb69a757':'gtk-gnutella 0.97', \
    'a746f9625f7695e8':'HeXHub 5.07', \
    '223bf0f360c6fea5':'I2P 0.8.8 (restartable)', \
    '2ff9dc8fb7e11f39':'I2P 0.8.8 (no window)', \
    'f1a4c04eebef2906':'[i2p] Robert 0.0.29 Preferences', \
    'c8e4c10e5460b00c':'iMesh 6.5.0.16898', \
    'f61b65550a84027e':'iMesh 11.0.0.112351', \
    'd460280b17628695':'Java Binary', \
    '784182360de0c5b6':'Kazaa Lite 1.7.1', \
    'a75b276f6e72cf2a':'Kazaa Lite Tools K++ 2.7.0', \
    'ba132e702c0147ef':'KCeasy 0.19-rc1', \
    'a8df13a46d66f6b5':'Kommute (Calypso) 0.24', \
    'c5ef839d8d1c76f4':'LimeWire 5.2.13', \
    '977a5d147aa093f4':'Lphant 3.51', \
    '96252daff039437a':'Lphant 7.0.0.112351', \
    'e76a4ef13fbf2bb1':'Manolito 3.1.1', \
    '99c15cf3e6d52b61':'mldonkey 3.1.0', \
    'ff224628f0e8103c':'Morpheus 3.0.3.6', \
    '792699a1373f1386':'Piolet 3.1.1', \
    'ca1eb46544793057':'RetroShare 0.5.2a (Build 4550)', \
    '3cf13d83b0bd3867':'RevConnect 0.674p (based on DC++)', \
    '5e01ecaf82f7d8e':'Scour Exchange 0.0.0.228', \
    '5d7b4175afdcc260':'Shareaza 2.0.0.0', \
    'b48ce76eda60b97':'Shareaza 8.0.0.112300', \
    '23f08dab0f6aaf30':'SoMud 1.3.3', \
    '135df2a440abe9bb':'SoulSeek 156c', \
    'ecd21b58c2f65a2f':'StealthNet 0.8.7.9', \
    '5ea2a50c7979fbdc':'TrustyFiles 3.1.0.22', \
    'cd8cafb0fb6afdab':'uTorrent 1.7.7 (Build 8179) / 1.8.5 / 2.0 / 2.21 (Build 25113) / 3.0 (Build 25583)', \
    '490c000889535727':'WinMX 4.9.3.0', \
    'ac3a63b839ac9d3a':'Vuze 4.6.0.4', \
    'd28ee773b2cea9b2':'3D-FTP 9.0 build 7', \
    'cd2acd4089508507':'AbsoluteTelnet 9.18 Lite', \
    'e6ef42224b845020':'ALFTP 5.20.0.4', \
    '9e0b3f677a26bbc4':'BitKinex 3.2.3', \
    '4cdf7858c6673f4b':'Bullet Proof FTP 1.26', \
    '714b179e552596df':'Bullet Proof FTP 2.4.0 (Build 31)', \
    '20ef367747c22564':'Bullet Proof FTP 2010.75.0.75', \
    '44a50e6c87bc012':'Classic FTP Plus 2.15', \
    '4fceec8e021ac978':'CoffeeCup Free FTP 3.5.0.0', \
    '8deb27dfa31c5c2a':'CoffeeCup Free FTP 4.4 (Build 1904)', \
    '49b5edbd92d8cd58':'FTP Commander 8.02', \
    '6a316aa67a46820b':'Core FTP LE 1.3c (Build 1437) / 2.2 (Build 1689)', \
    'be4875bb3e0c158f':'CrossFTP 1.75a', \
    'c04f69101c131440':'CuteFTP 5.0 (Build 50.6.10.2)', \
    'a79a7ce3c45d781':'CuteFTP 7.1 (Build 06.06.2005.1)', \
    '59e86071b87ac1c3':'CuteFTP 8.3 (Build 8.3.4.0007)', \
    'd8081f151f4bd8a5':'CuteFTP 8.3 Lite (Build 8.3.4.0007)', \
    '3198e37206f28dc7':'CuteFTP 8.3 Professional (Build 8.3.4.0007)', \
    'f82607a219af2999':'Cyberduck 4.1.2 (Build 8999)', \
    'fa7144034d7d083d':'Directory Opus 10.0.2.0.4269 (JL tasks supported)', \
    'f91fd0c57c4fe449':'ExpanDrive 2.1.0', \
    '8f852307189803b8':'Far Manager 2.0.1807', \
    '226400522157fe8b':'FileZilla Server 0.9.39 beta', \
    'a1d19afe5a80f80':'FileZilla 2.2.32', \
    'e107946bb682ce47':'FileZilla 3.5.1', \
    'b7cb1d1c1991accf':'FlashFXP 4.0.0 (Build 1548)', \
    '8628e76fd9020e81':'Fling File Transfer Plus 2.24', \
    '27da120d7e75cf1f':'pbFTPClient 6.1', \
    'f64de962764b9b0f':'FTPRush 1.1.3 / 2.15', \
    '10f5a20c21466e85':'FTP Voyager 15.2.0.17', \
    '7937df3c65790919':'FTP Explorer 10.5.19 (Build 001)', \
    '9560577fd87cf573':'LeechFTP 1.3 (Build 207)', \
    'fc999f29bc5c3560':'Robo-FTP 3.7.9', \
    'c99ddde925d26df3':'Robo-FTP 3.7.9 CronMaker', \
    '4b632cf2ceceac35':'Robo-FTP Server 3.2.5', \
    '3a5148bf2288a434':'Secure FTP 2.6.1 (Build 20101209.1254)', \
    '435a2f986b404eb7':'SmartFTP 4.0.1214.0', \
    'e42a8e0f4d9b8dcf':'Sysax FTP Automation 5.15', \
    'b8c13a5dd8c455a2':'Titan FTP Server 8.40 (Build 1338)', \
    '7904145af324576e':'Total Commander 7.56a (Build 16.12.2010)', \
    '79370f660ab51725':'UploadFTP 2.0.1.0', \
    '6a8b377d0f5cb666':'WinSCP 2.3.0 (Build 146)', \
    '9a3bdae86d5576ee':'WinSCP 3.2.1 (Build 174) / 3.8.0 (Build 312)', \
    '6bb54d82fa42128d':'WinSCP 4.3.4 (Build 1428)', \
    'b6267f3fcb700b60':'WiseFTP 4.1.0', \
    'a581b8002a6eb671':'WiseFTP 5.5.9', \
    '2544ff74641b639d':'WiseFTP 6.1.5', \
    'c54b96f328bdc28d':'WiseFTP 7.3.0', \
    'b223c3ffbc0a7a42':'Bersirc 2.2.14', \
    'c01d68e40226892b':'ClicksAndWhistles 2.7.146', \
    'ac8920ed05001800':'DMDirc 0.6.5 (Profile store: C:\Users\$user\AppData\Roaming\DMDirc\)', \
    'd3530c5294441522':'HydraIRC 0.3.165', \
    '8904a5fd2d98b546':'IceChat 7.70 20101031', \
    '6b3a5ce7ad4af9e4':'IceChat 9 RC2', \
    'fa496fe13dd62edf':'KVIrc 3.4.2.1 / 4.0.4', \
    '65f7dd884b016ab2':'LimeChat 2.39', \
    '19ccee0274976da8':'mIRC 4.72 / 5.61', \
    'ae069d21df1c57df':'mIRC 6.35 / 7.19', \
    'e30bbea3e1642660':'Neebly 1.0.4', \
    '54c803dfc87b52ba':'Nettalk 6.7.12', \
    'dd658a07478b46c2':'PIRCH98 1.0.1.1190', \
    '6fee01bd55a634fe':'Smuxi 0.8.0.0', \
    '2a5a615382a84729':'X-Chat 2 2.8.6-2', \
    'b3965c840bf28ef4':'AIM 4.8.2616', \
    '1b29f0dc90366bb':'AIM 5.9.3857', \
    '27ececd8d89b6767':'AIM 6.2.14.2 / 6.5.3.12 / 6.9.17.2', \
    '6f647f9488d7a':'AIM 7.5.11.9 (custom AppID + JL support)', \
    'ca942805559495e9':'aMSN 0.98.4', \
    'c6f7b5bf1b9675e4':'BitWise IM 1.7.3a', \
    'fb1f39d1f230480a':'Bopup Messenger 5.6.2.9178 (all languages: en;du;fr;ger;rus;es)', \
    'dc64de6c91c18300':'Brosix Communicator 3.1.3 (Build 110719 nid 1)', \
    'f09b920bfb781142':'Camfrog 4.0.47 / 5.5.0 / 6.1 (build 146) (JL support)', \
    'ebd8c95d87f25154':'Carrier 2.5.5', \
    '30d23723bdd5d908':'Digsby (Build 30140) (JL support)', \
    '728008617bc3e34b':'eM Client 3.0.10206.0', \
    '689319b6547cda85':'emesene 2.11.7', \
    '454ef7dca3bb16b2':'Exodus 0.10.0.0', \
    'cca6383a507bac64':'Gadu-Gadu 10.5.2.13164', \
    '4278d3dc044fc88a':'Gaim 1.5.0', \
    '777483d3cdac1727':'Gajim 0.14.4', \
    '6aa18a60024620ae':'GCN 2.9.1', \
    '3f2cd46691bbee90':'GOIM 1.1.0', \
    '73c6a317412687c2':'Google Talk 1.0.0.104', \
    'b0236d03c0627ac4':'ICQ 5.1 / ICQLite Build 1068', \
    'a5db18f617e28a51':'ICQ 6.5 (Build 2024)', \
    '2417caa1f2a881d4':'ICQ 7.6 (Build 5617)', \
    '989d7545c2b2e7b2':'IMVU 465.8.0.0', \
    'a3e0d98f5653b539':'Instantbird 1.0 (20110623121653) (JL support)', \
    'bcc705f705d8132b':'Instan-t 5.2 (Build 2824)', \
    '6059df4b02360af':'Kadu 0.10.0 / 0.6.5.5', \
    'c312e260e424ae76':'Mail.Ru Agent 5.8 (JL support)', \
    '22cefa022402327d':'Meca Messenger 5.3.0.52', \
    '86b804f7a28a3c17':'Miranda IM 0.6.8 / 0.7.6 / 0.8.27 / 0.9.9 / 0.9.29 (ANSI + Unicode)', \
    'b868d9201b866d96':'Microsoft Lync 4.0.7577.0', \
    '8c816c711d66a6b5':'MSN Messenger 6.2.0137 / 7.0.0820', \
    '2d1658d5dc3cbe2d':'MySpaceIM 1.0.823.0 Beta', \
    'bf9ae1f46bd9c491':'Nimbuzz 2.0.0 (rev 6266)', \
    'fb7ca8059b8f2123':'ooVoo 3.0.7.21', \
    'efb08d4e11e21ece':'Paltalk Messenger 10.0 (Build 409)', \
    '4f24a7b84a7de5a6':'Palringo 2.6.3 (r45983)', \
    'e93dbdcede8623f2':'Pandion 2.6.106', \
    'aedd2de3901a77f4':'Pidgin 2.0.0 / 2.10.0 / 2.7.3', \
    'c5236fd5824c9545':'PLAYXPERT 1.0.140.2822', \
    'dee18f19c7e3a2ec':'PopNote 5.21', \
    '1a60b1067913516a':'Psi 0.14', \
    'e0532b20aa26a0c9':'QQ International 1.1 (2042)', \
    '3c0022d9de573095':'QuteCom 2.2', \
    '93b18adf1d948fa3':'qutIM 0.2', \
    'e0246018261a9ccc':'qutIM 0.2.80.0', \
    '2aa756186e21b320':'RealTimeQuery 3.2', \
    '521a29e5d22c13b4':'Skype 1.4.0.84 / 2.5.0.154 / 3.8.0.139 / 4.2.0.187 / Skype 5.3.0.120 / 5.5.0.115 / 5.5.32.117', \
    '70b52cf73249257':'Sococo 1.5.0.2274', \
    'd41746b133d17456':'Tkabber 0.11.1', \
    'c8aa3eaee3d4343d':'Trillian 0.74 / 3.1 / 4.2.0.25 / 5.0.0.35 (JL support)', \
    'd7d647c92cd5d1e6':'uTalk 2.6.4 r47692', \
    '36c36598b08891bf':'Vovox 2.5.3.4250', \
    '884fd37e05659f3a':'VZOchat 6.3.5', \
    '3461e4d1eb393c9c':'WTW 0.8.18.2852 / 0.8.19.2940', \
    'f2cb1c38ab948f58':'X-Chat 1.8.10 / 2.6.9 / 2.8.9', \
    '4e0ac37db19cba15':'Xfire 1.138 (Build 44507)', \
    'da7e8de5b8273a0f':'Yahoo Messenger 5.0.0.1226 / 6.0.0.1922', \
    '62dba7fb39bb0adc':'Yahoo Messenger 7.5.0.647 / 8.1.0.421 / 9.0.0.2162 / 10.0.0.1270', \
    'fb230a9fe81e71a8':'Yahoo Messenger 11.0.0.2014-us', \
    'b06a975b62567622':'Windows Live Messenger 8.5.1235.0517 BETA', \
    'bd249197a6faeff2':'Windows Live Messenger 2011', \
    'd22ad6d9d20e6857':'ALLPlayer 4.7', \
    '7494a606a9eef18e':'Crystal Player 1.98', \
    '1cffbe973a437c74':'DSPlayer 0.889 Lite', \
    '817bb211c92fd254':'GOM Player 2.0.12.3375 / 2.1.28.5039', \
    '6bc3383cb68a3e37':'iTunes 7.6.0.29 / 8.0.0.35', \
    '83b03b46dcd30a0e':'iTunes 9.0.0.70 / 9.2.1.5 / 10.4.1.10 (begin custom "Tasks" JL capability)', \
    'fe5e840511621941':'JetAudio 5.1.9.3018 Basic / 6.2.5.8220 Basic / 7.0.0 Basic / 8.0.16.2000 Basic', \
    'a777ad264b54abab':'JetVideo 8.0.2.200 Basic', \
    '3c93a049a30e25e6':'J. River Media Center 16.0.149', \
    '4a49906d074a3ad3':'Media Go 1.8 (Build 121)', \
    '1cf97c38a5881255':'MediaPortal 1.1.3', \
    '62bff50b969c2575':'Quintessential Media Player 5.0 (Build 121) - also usage stats (times used, tracks played, total time used)', \
    'b50ee40805bd280f':'QuickTime Alternative 1.9.5 (Media Player Classic 6.4.9.1)', \
    'ae3f2acd395b622e':'QuickTime Player 6.5.1 / 7.0.3 / 7.5.5 (Build 249.13)', \
    '7593af37134fd767':'RealPlayer 6.0.6.99 / 7 / 8 / 10.5', \
    '37392221756de927':'RealPlayer SP 12', \
    'f92e607f9de02413':'RealPlayer 14.0.6.666', \
    '6e9d40a4c63bb562':'Real Player Alternative 1.25 (Media Player Classic 6.4.8.2 / 6.4.9.0)', \
    'c91d08dcfc39a506':'SM Player 0.6.9 r3447', \
    'e40cb5a291ad1a5b':'Songbird 1.9.3 (Build 1959)', \
    '4d8bdacf5265a04f':'The KMPlayer 2.9.4.1434', \
    '4acae695c73a28c7':'VLC 0.3.0 / 0.4.6', \
    '9fda41b86ddcf1db':'VLC 0.5.3 / 0.8.6i / 0.9.7 / 1.1.11', \
    'e6ee34ac9913c0a9':'VLC 0.6.2', \
    'cbeb786f0132005d':'VLC 0.7.2', \
    'f674c3a77cfe39d0':'Winamp 2.95 / 5.1 / 5.621', \
    '90e5e8b21d7e7924':'Winamp 3.0d (Build 488)', \
    '74d7f43c1561fc1e':'Windows Media Player 12.0.7601.17514 x86', \
    'ed7a5cc3cca8d52a':'CCleaner 1.32.345 / 1.41.544 / 2.36.1233 / 3.10.1525', \
    'eb7e629258d326a1':'WindowWasher 6.6.1.18', \
    'ace8715529916d31':'40tude Dialog 2.0.15.1 (Beta 38)', \
    'cc76755e0f925ce6':'AllPicturez 1.2', \
    '36f6bc3efe1d99e0':'Alt.Binz 0.25.0 (Build 27.09.2007)', \
    'd53b52fb65bde78c':'Android Newsgroup Downloader 6.2', \
    'c845f3a6022d647c':'Another File 2.03 (Build 2/7/2004)', \
    '780732558f827a42':'AutoPix 5.3.3', \
    'baea31eacd87186b':'BinaryBoy 1.97 (Build 55)', \
    'eab25958dbddbaa4':'Binary News Reaper 2 (Beta 0.14.7.448)', \
    'bf483b423ebbd327':'Binary Vortex 5.0', \
    '36801066f71b73c5':'Binbot 2.0', \
    '13eb0e5d9a49eaef':'Binjet 3.0.2', \
    '8172865a9d5185cb':'Binreader 1.0 (Beta 1)', \
    '6224453d9701a612':'BinTube 3.7.1.0 (requires VLC 10.5!)', \
    'cf6379a9a987366e':'Digibin 1.31', \
    '43886ba3395acdcc':'Easy Post 3.0', \
    'cfab0ec14b6f953':'Express NewsPictures 2.41 (Build 08.05.07.0)', \
    '7526de4a8b5914d9':'Forte Agent 6.00 (Build 32.1186)', \
    'c02baf50d02056fc':'FotoVac 1.0', \
    '3ed70ef3495535f7':'Gravity 3.0.4', \
    '86781fe8437db23e':'Messenger Pro 2.66.6.3353', \
    'f920768fe275f7f4':'Grabit 1.5.3 Beta (Build 909) / 1.6.2 (Build 940) / 1.7.2 Beta 4 (Build 997)', \
    '9f03ae476ad461fa':'GroupsAloud 1.0', \
    'd0261ed6e16b200b':'News File Grabber 4.6.0.4', \
    '8211531a7918b389':'Newsbin Pro 6.00 (Build 1019) (JL support)', \
    'd1fc019238236806':'Newsgroup Commander Pro 9.05', \
    '186b5ccada1d986b':'NewsGrabber 3.0.36', \
    '4d72cfa1d0a67418':'Newsgroup Image Collector', \
    '92f1d5db021cd876':'NewsLeecher 4.0 / 5.0 Beta 6', \
    'd7666c416cba240c':'NewsMan Pro 3.0.5.2', \
    '7b2b4f995b54387d':'News Reactor 20100224.16', \
    'cb984e3bc7faf234':'NewsRover 17.0 (Rev.0)', \
    'c98ab5ccf25dda79':'NewsShark 2.0', \
    'dba909a61476ccec':'NewsWolf 1.41', \
    '2b164f512891ae37':'NewsWolf NSListGen', \
    'cb1d97aca3fb7e6b':'Newz Crawler 1.9.0 (Build 4100)', \
    '3be7b307dfccb58f':'NiouzeFire 0.8.7.0', \
    'de76415e0060ce13':'Noworyta News Reader 2.9', \
    'cd40ead0b1eb15ab':'NNTPGrab 0.6.2', \
    'd5c02fc7afbb3fd4':'NNTPGrab 0.6.2 Server', \
    'a4def57ee99d77e9':'Nomad News 1.43', \
    '3f97341a65bac63a':'Ozum 6.07 (Build 6070)', \
    'bfe841f4d35c92b1':'QuadSucker/News 5.0', \
    'd3c5cf21e86b28af':'SeaMonkey 2.3.3', \
    '7a7c60efd66817a2':'Spotnet 1.7.4', \
    'eb3300e672136bc7':'Stream Reactor 1.0 Beta 9 (uses VLC!)', \
    '3168cc975b354a01':'Slypheed 3.1.2 (Build 1120)', \
    '776beb1fcfc6dfa5':'Thunderbird 1.0.6 (20050716) / 3.0.2', \
    '3d877ec11607fe4':'Thunderbird 6.0.2', \
    '7192f2de78fd9e96':'TIFNY 5.0.3', \
    '9dacebaa9ac8ca4e':'TLNews Newsreader 2.2.0 (Build 2430)', \
    '7fd04185af357bd5':'UltraLeeacher 1.7.0.2969 / 1.8 Beta (Build 3490)', \
    'aa11f575087b3bdc':'Unzbin 2.6.8', \
    'd7db75db9cdd7c5d':'Xnews 5.04.25', \
    '3dc02b55e44d6697':'7-Zip 3.13 / 4.20', \
    '4975d6798a8bdf66':'7-Zip 4.65 / 9.20', \
    '4b6925efc53a3c08':'BCWipe 5.02.2 Task Manager 3.02.3', \
    '23709f6439b9f03d':'Hex Editor Neo 5.14', \
    'e57cfc995bdc1d98':'Snagit 11', \
    '337ed59af273c758':'Sticky Notes', \
    '290532160612e071':'WinRAR 2.90 / 3.60 / 4.01 x64', \
    'c9950c443027c765':'WinZip 9.0 SR-1 (6224) / 10.0 (6667)', \
    'b74736c2bd8cc8a5':'WinZip 15.5 (9468)', \
    'bc0c37e84e063727':'Windows Command Processor - cmd.exe x86', \
    '2807f361e520f69b':'KaKaoTalk v1.1.8.546 x86', \
    '382149bb0c9e9ae4':'KaKaoTalk v1.1.8.546 x64', \
    '20f18d57e149e379':'Hancom Office Hanword 2010', \
    '284527bafaf6fe2e':'Daum PotPlayer x86', \
    '38639d6013489251':'Daum PotPlayer x64'
    }

# result filename stored
resultFileName = ''

# ShellLinkHeader Structure
shellLinkHeader = dict(\
    HeaderSize=0, \
    FileAttributes=0, \
    CreationTimeH=0, \
    CreationTimeL=0, \
    ModfiedTimeH=0, \
    ModifiedTimeL=0, \
    AccessTimeH=0, \
    AccessTimeL=0, \
    FileSize=0, \
    ShowCommand=0, \
    sLinkFlags=0 \
    )

# LinkInfo Structure
LinkInfo = dict(\
    LinkInfoSize=0, \
    LocalBasePath='', \
    VolumeIDOffset=0, \
    LinkInfoHeaderSize=0 \
    )

# VolumeID Structure
VolumeID = dict(\
    VolumeIDSize=0, \
    DriveType=0, \
    DriveSerialNumber=0 \
    )

# sLinkTargetIDList Structure
sLinkTargetIDList = dict(\
    LinkTargetIDListSize=0, \
    LocalBasePathOffset=0, \
    TerminalID=0 \
    )

# DestList Structure
DestList = dict(\
    CheckSum=0, \
    NewVolumeID=0, \
    ObjectID_1=0, \
    BirthVolumeID=0, \
    ObjectID_2=0, \
    MachineID=0, \
    EntryID=0, \
    FileAccessCount=0, \
    LastRecordTimeL=0, \
    LastRecordTimeH=0, \
    pin=0, \
    UnicodeStringLength=0, \
    StringData=0 \
    )

# Drive Type
DriveType = (\
    'DRIVE_NUKNOWN', \
    'DRIVE_NO_ROOT_DIR', \
    'DRIVE_REMOVABLE', \
    'DRIVE_FIXED', \
    'DRIVE_REMOTE', \
    'DRIVE_CDROM', \
    'DRIVE_RAMDISK' \
    )

# File(Stream) Attribute List
File_attr = (\
    'UNKNOWN', \
    'ENCRYPTED', \
    'NOT_CONTENT', \
    'OFFLINE', \
    'COMPRESSED', \
    'REPARSE_POINT', \
    'SPARSE_FILE', \
    'TEMPORARY', \
    'NORMAL', \
    'ARCHIVE', \
    'DIRECTORY', \
    'UNKNOWN', \
    'SYSTEM', \
    'HIDDEN', \
    'READONLY', \
    )

# Tracker ExtraData Structure
TrackerData = dict(\
    BlockSize=0, \
    Length=0, \
    MachineID='', \
    MacAddress=0, \
    TerminalID=0 \
    )

# COMMAND LINE ARGUMENT Structure
commandLine = dict(\
    length = 0, \
    StringData = '' \
    )

# ICON LOCATION Structure
iconLocation = dict( \
    length = 0, \
    StringData = ''\
    )

# RELATIVE PATH Structure
RelativePath = dict( \
    length=0, \
    StringData='' \
    )

# WORKING DIR Structure
WorkingDir = dict( \
    length=0, \
    StringData='' \
    )

# lnk Description
Name = dict( \
    length=0, \
    StringData='' \
    )

# AutomationDestinations JumpList analysis output function
def auto_result_print(filename, streamName, sLinkTargetIDList, LinkInfo, Name, RelativePath, WorkingDir, commandLine, iconLocation, VolumeID, TrackerData, sLinkTargetIDList_CLSID, ExtraDataBlock_SID, output):
   # global
   global resultFileName

   # Stream Attributes Check
   attr_list = list()
   if len(shellLinkHeader["FileAttributes"]) < 15:
       for j in range(15-len(shellLinkHeader["FileAttributes"])):
           shellLinkHeader["FileAttributes"] = '0' + shellLinkHeader["FileAttributes"]

   for i in range(len(shellLinkHeader["FileAttributes"])):
       if shellLinkHeader["FileAttributes"][i] == '1':
           attr_list.append(File_attr[i])
   stream_attr = '|'.join(attr_list)

   # Drive Type Check
   d_type = DriveType[VolumeID["DriveType"]]

   # converting Time
   conv_cTime = conv_time(shellLinkHeader["CreationTimeL"], shellLinkHeader["CreationTimeH"])
   conv_mTime = conv_time(shellLinkHeader["ModifiedTimeL"], shellLinkHeader["ModifiedTimeH"])
   conv_aTime = conv_time(shellLinkHeader["AccessTimeL"], shellLinkHeader["AccessTimeH"])

   # commandLine, iconLocation, Name, Relative, WorkingDir char remove
   commandLine["StringData"] = commandLine["StringData"].replace('\x00', '').replace('\r', '').replace('\n', '').replace(',', ' ')
   iconLocation["StringData"] = iconLocation["StringData"].replace('\x00', '').replace('\r', '').replace('\n', '').replace(',', ' ')
   Name["StringData"] = Name["StringData"].replace('\x00', '').replace('\r', '').replace('\n', '').replace(',', ' ')
   RelativePath["StringData"] = RelativePath["StringData"].replace('\x00', '').replace('\r', '').replace('\n', '').replace(',', ' ')
   WorkingDir["StringData"] = WorkingDir["StringData"].replace('\x00', '').replace('\r', '').replace('\n', '').replace(',', ' ')

   # MAC Address Assembly
   MAC_Address = ''.join((TrackerData["MacAddress"][0:2], "-", TrackerData["MacAddress"][2:4], "-", TrackerData["MacAddress"][4:6], "-", TrackerData["MacAddress"][6:8], \
                          "-", TrackerData["MacAddress"][8:10], "-", TrackerData["MacAddress"][10:12]))

   IDList_CSIDL = ''.join(sLinkTargetIDList_CLSID).replace(",", "").replace("\n", "").replace("\x00", "").replace("\r", "")
   ExtraData_SID = ''.join(ExtraDataBlock_SID).replace(",", "").replace("\n", "").replace("\x00", "").replace("\r", "")

   if output == 'display':
       if streamName != "DestList":
           print "\n"
           print " JumpList File Name : \t\t", filename
           print " Stream Name : \t\t\t", streamName
           print " Stream Attributes : \t\t", stream_attr
           print " Stream Created Time(Local) : \t", conv_cTime
           print " Stream Modified Time(Local) : \t", conv_mTime
           print " Stream Accessed Time(Local) : \t", conv_aTime
           print " Drive Type : \t\t\t", d_type
           print " Stream Size : \t\t\t", shellLinkHeader["FileSize"]
           print " Volume Serial Number : \t", VolumeID["DriveSerialNumber"]
           print " Local Base Path : \t\t", LinkInfo["LocalBasePath"]
           print " Relative Path : \t\t\t", RelativePath["StringData"]
           print " Working Directory : \t\t\t", WorkingDir["StringData"]
           print " File Description : \t\t\t", Name["StringData"]
           print " Command Arguments : \t\t", commandLine["StringData"]
           print " Icon Location : \t\t", iconLocation["StringData"]
           print " NetBIOS Name : \t\t", TrackerData["MachineID"]
           print " MAC Address : \t\t\t", MAC_Address
           print " ItemIDList : \t\t\t", IDList_CSIDL
           print " ExtraData : \t\t\t", ExtraData_SID

   elif output == 'csv':
       if streamName != "DestList":
           f = open(resultFileName+"_stream.csv", 'a')
           f.write(filename+","+streamName+","+stream_attr+","+conv_cTime+","+conv_mTime+","+conv_aTime+","+d_type+","+str(shellLinkHeader["FileSize"])+","+str(VolumeID["DriveSerialNumber"])+","+LinkInfo["LocalBasePath"]+","+\
                   RelativePath["StringData"]+","+WorkingDir["StringData"]+","+Name["StringData"]+","+str(commandLine["StringData"])+","+str(iconLocation["StringData"])+","+TrackerData["MachineID"]+","+MAC_Address+","+IDList_CSIDL+","+ExtraData_SID+"\n")
           f.close()

   elif output == 'sqlite':
       con = sqlite3.connect(resultFileName+"_automatic.db")
       if streamName != "DestList":
           try:
               cur = con.cursor()
               query = "INSERT INTO stream(\
               JumpListFileName, \
               StreamName, \
               StreamAttributes, \
               StreamCreatedTime, \
               StreamModifiedTime, \
               StreamAccessedTime, \
               DriveType, \
               StreamSize, \
               VolumeSerialNumber, \
               LocalBasePath, \
               RelativePath, \
               WorkingDirectory, \
               FileDescription, \
               CommandArguments, \
               IconLocation, \
               NetBIOSName, \
               MACAddress, \
               ItemIDList, \
               ExtraData) \
               VALUES(\"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\")"%(\
                   filename, streamName, stream_attr, conv_cTime,  conv_mTime, conv_aTime, d_type, str(shellLinkHeader["FileSize"]), str(VolumeID["DriveSerialNumber"]), \
                   LinkInfo["LocalBasePath"], RelativePath["StringData"], WorkingDir["StringData"], Name["StringData"], commandLine["StringData"], iconLocation["StringData"], TrackerData["MachineID"], MAC_Address, IDList_CSIDL.split("\"")[0], ExtraData_SID)
               cur.execute(query)
               con.commit()
           except:
               pass


# Time Converting function
def conv_time(low, high):
    '''
    @low left 4byte of 8byte
    @high right 4byte of 8byte
    '''
    default = 116444736000000000L

    if low+high != 0:
        conv_time = (((long(high) << 32) + long(low)) - default)/10000000
    else:
        conv_time = 0

    return time.strftime("%Y/%m/%d %H:%M:%S %a", time.localtime(conv_time))

# ExtraBlocks SID Parsing function
def SID_Extra(data):
    '''
    @data ExtraBlock Structure data
    '''
    temp = ''
    blockSize = 0
    offset = 0
    SID = list()
    count = 0

    while data[offset+4:offset+8] != '\03\x00\x00\xA0':
        if count >= 7:
            break
        count += 1
        blockSize = struct.unpack('<l', binascii.unhexlify(data[offset+0:offset+4].encode('hex')))[0]
        signature = data[offset+4:offset+8]
        if  signature == '\x09\x00\x00\xA0':
            dataSize = struct.unpack('<l', binascii.unhexlify(data[offset+8:offset+12].encode('hex')))[0]
            SID.append(data[offset+12:offset+12+dataSize])
        if signature == '\x01\x00\x00\xA0':
            SID.append(data[offset+8:offset+blockSize-4])
        offset += blockSize
    return count, SID

# sLinkTargetIDList Parsing function
def sLinkTargetIDList_func(data):
    '''
    @data sLinkTargetIDList Structure data
    '''
    offset = 0
    count = 0
    temp = 0
    ItemIDSize = 0
    CLSID = list()

    while data[offset:offset+2] != '\x00\x00':
        count += 1
        ItemIDSize = struct.unpack('<H', binascii.unhexlify(data[offset:offset+2].encode('hex')))[0]
        temp = data[offset+2:offset+ItemIDSize]
        CLSID.append(temp)
        offset += ItemIDSize

    return count, CLSID

# DestList List Parsing and output
def DestList_func(filename, data, output_type):
    '''
    @filename DestList in filename
    @data DestList data
    @output_type output type [display|csv|sqlite]
    '''
    global resultFileName
    start_offset = 32
    next_offset = 0
    while 1:
        try:
            DestList["CheckSum"] = data[start_offset:start_offset+8].encode('hex')
            DestList["NewVolumeID"] = data[start_offset+8:start_offset+24].encode('hex')
            DestList["ObjectID_1"] = data[start_offset+24:start_offset+40].encode('hex')
            DestList["BirthVolumeID"] = data[start_offset+40:start_offset+56].encode('hex')
            DestList["ObjectID_2"] = data[start_offset+56:start_offset+72].encode('hex')
            DestList["MachineID"] = data[start_offset+72:start_offset+88].replace("\x00", "")
            DestList["EntryID"] = hex(struct.unpack('<l', data[start_offset+88:start_offset+92])[0]).replace("0x", "")
            DestList["FileAccessCount"] = struct.unpack('!f', data[start_offset+96:start_offset+100])[0]
            DestList["LastRecordTimeL"] = struct.unpack("<l", binascii.unhexlify(data[start_offset+100:start_offset+104].encode('hex')))[0]
            DestList["LastRecordTimeH"] = struct.unpack("<l", binascii.unhexlify(data[start_offset+104:start_offset+108].encode('hex')))[0]
            DestList["pin"] = data[start_offset+108:start_offset+112].encode('hex')
            DestList["UnicodeStringLength"] = struct.unpack('<H', binascii.unhexlify(data[start_offset+112:start_offset+114].encode('hex')))[0]

            conv_rTime = conv_time(DestList["LastRecordTimeL"], DestList["LastRecordTimeH"])

            # Extract String data
            if DestList["UnicodeStringLength"] != -1:
                stringData_offset = start_offset+114
                DestList["StringData"] = data[stringData_offset:stringData_offset+DestList["UnicodeStringLength"]*2].replace("\x00", "")
                next_offset = stringData_offset+DestList["UnicodeStringLength"]*2

            # Next Block starting offset setting
            start_offset = next_offset

            # output type is display
            if output_type == 'display':
                # output
                print "\n [DestList]"
                print " JumpList File Name : \t", filename
                print " Stream Name : \t\t", DestList["EntryID"]
                print " Check Sum : \t\t", DestList["CheckSum"]
                print " New Volume ID : \t", DestList["NewVolumeID"]
                print " Object ID_1 : \t\t", DestList["ObjectID_1"]
                print " Object ID_2 : \t\t", DestList["ObjectID_2"]
                print " Birth Volume ID : \t", DestList["BirthVolumeID"]
                print " Machine ID : \t\t", DestList["MachineID"]
                print " File Access Count : \t", DestList["FileAccessCount"]
                print " Last Record Time : \t", conv_rTime
                print " String Data : \t\t", DestList["StringData"]

            # output type is csv
            if output_type == 'csv':
                # stored File
                fp = open(resultFileName+"_destlist.csv", 'a')
                fp.write(filename+","+","+str(DestList["EntryID"])+","+str(DestList["CheckSum"])+","+DestList["NewVolumeID"]+","+DestList["ObjectID_1"]+","+DestList["ObjectID_2"]+\
                         ","+DestList["BirthVolumeID"]+","+DestList["MachineID"]+","+str(DestList["FileAccessCount"])+","+conv_rTime+","+DestList["StringData"]+"\n")
                fp.close()

            # output type is sqlite
            if output_type == 'sqlite':
                try:
                    con = sqlite3.connect(resultFileName+"_automatic.db")
                    cur = con.cursor()
                    query = "INSERT INTO destlist(\
                    JumpListFileName, \
                    StreamName, \
                    CheckSum, \
                    NewVolumeID, \
                    ObjectID_1, \
                    ObjectID_2, \
                    BirthVolumeID, \
                    MachineID, \
                    FileAccessCount, \
                    LastRecordTime, \
                    StringData) \
                    VALUES(\"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\")"%(\
                        filename, DestList["EntryID"], str(DestList["CheckSum"]), DestList["NewVolumeID"],  DestList["ObjectID_1"], DestList["ObjectID_2"], DestList["BirthVolumeID"], DestList["MachineID"], \
                        DestList["FileAccessCount"], conv_rTime, DestList["StringData"])
                    cur.execute(query)
                    con.commit()
                except:
                    pass

        except:
            break

#Stream in AutomationDestination LnkParsing
def LnkParser(f_name, s_Name, data, output_type):
    '''
    @f_name Filename
    @Name Streamname
    @data stream data
    @output_type result output type [display|csv|sqlite]
    '''

    sLinkTargetIDList_offset = 0
    sLinkTargetIDList_BlockSize = 0
    LinkInfo_offset = 0
    LinkInfo_BlockSize = 0
    Name_offset = 0
    Name_BlockSize = 0
    RelativePath_offset = 0
    RelativePath_BlockSize = 0
    WorkingDir_offset = 0
    WorkingDir_BlockSize = 0
    commandLine_offset = 0
    commandLine_BlockSize = 0
    iconLocation_offset = 0
    iconLocation_BlockSize = 0
    ExtraData_offset = 0
    TrackerData_offset = 0
    sLinkTargetIDList_count = 0
    sLinkTargetIDList_CLSID = list()
    ExtraDataBlock_count = 0
    ExtraDataBlock_SID = list()

    # DestList Result stored
    DestList_func_result = list()

    try:
        # Stream Parsing
        if s_Name != "DestList":
           # ShellLinkHeader Parsing
            shellLinkHeader["HeaderSize"] = struct.unpack('<L', binascii.unhexlify(data[0:4].encode('hex')))[0]
            shellLinkHeader["FileAttributes"] = "{0:b}".format(struct.unpack("<HH", binascii.unhexlify(data[24:28].encode('hex')))[0])
            shellLinkHeader["CreationTimeL"] = struct.unpack("<l", binascii.unhexlify(data[28:32].encode('hex')))[0]
            shellLinkHeader["CreationTimeH"] = struct.unpack("<l", binascii.unhexlify(data[32:36].encode('hex')))[0]
            shellLinkHeader["ModifiedTimeL"] = struct.unpack("<l", binascii.unhexlify(data[36:40].encode('hex')))[0]
            shellLinkHeader["ModifiedTimeH"] = struct.unpack("<l", binascii.unhexlify(data[40:44].encode('hex')))[0]
            shellLinkHeader["AccessTimeL"] = struct.unpack("<l", binascii.unhexlify(data[44:48].encode('hex')))[0]
            shellLinkHeader["AccessTimeH"] = struct.unpack("<l", binascii.unhexlify(data[48:52].encode('hex')))[0]
            shellLinkHeader["FileSize"] = struct.unpack('<L', binascii.unhexlify(data[52:56].encode('hex')))[0]
            shellLinkHeader["ShowCommand"] = struct.unpack('<l', binascii.unhexlify(data[60:64].encode('hex')))[0]
            shellLinkHeader["sLinkFlags"] = "{0:b}".format(struct.unpack("<HH", binascii.unhexlify(data[20:24].encode('hex')))[0])

            # Link Flags Check
            if shellLinkHeader["sLinkFlags"][-1] == '1': # HasLinkTargetIDList True
                sLinkTargetIDList_offset = shellLinkHeader["HeaderSize"]
                sLinkTargetIDList["LinkTargetIDListSize"] = struct.unpack('<H', binascii.unhexlify(data[sLinkTargetIDList_offset:sLinkTargetIDList_offset+2].encode('hex')))[0]
                sLinkTargetIDList_BlockSize = sLinkTargetIDList["LinkTargetIDListSize"]+2

                # sLinkTargetIDList Parsing
                sLinkTargetIDList_count, sLinkTargetIDList_CLSID = sLinkTargetIDList_func(data[sLinkTargetIDList_offset:sLinkTargetIDList_offset+sLinkTargetIDList["LinkTargetIDListSize"]+2])
                sLinkTargetIDList["TerminalID"] = data[sLinkTargetIDList_offset+sLinkTargetIDList["LinkTargetIDListSize"]-2:sLinkTargetIDList_offset+sLinkTargetIDList["LinkTargetIDListSize"]].encode('hex')

            if shellLinkHeader["sLinkFlags"][-2] == '1': # HasLinkInfo True
                LinkInfo_offset = shellLinkHeader["HeaderSize"]+sLinkTargetIDList_BlockSize
                LinkInfo["LinkInfoSize"] = struct.unpack('<l', binascii.unhexlify(data[LinkInfo_offset:LinkInfo_offset+4].encode('hex')))[0]
                LinkInfo_BlockSize = LinkInfo["LinkInfoSize"]

                # LinkInfo Parsing
                LinkInfo["LinkInfoHeaderSize"] = struct.unpack('<l', binascii.unhexlify(data[LinkInfo_offset+4:LinkInfo_offset+8].encode('hex')))[0]
                LinkInfo["VolumeIDOffset"] = struct.unpack('<l', binascii.unhexlify(data[LinkInfo_offset+12:LinkInfo_offset+16].encode('hex')))[0]
                LinkInfo["LocalBasePathOffset"] = struct.unpack('<l', binascii.unhexlify(data[LinkInfo_offset+16:LinkInfo_offset+20].encode('hex')))[0]
                LinkInfo["LocalBasePath"] = data[LinkInfo_offset+LinkInfo["LocalBasePathOffset"]:LinkInfo_offset+LinkInfo["LinkInfoSize"]-2]

                # VolumeID Parsing
                VolumeID["VolumeSize"] = struct.unpack('<l', binascii.unhexlify(data[LinkInfo_offset+28:LinkInfo_offset+32].encode('hex')))[0]
                VolumeID["DriveType"] = struct.unpack('<l', binascii.unhexlify(data[LinkInfo_offset+32:LinkInfo_offset+36].encode('hex')))[0]
                VolumeID["DriveSerialNumber"] =struct.unpack('<L', binascii.unhexlify(data[LinkInfo_offset+36:LinkInfo_offset+40].encode('hex')))[0]

            if shellLinkHeader["sLinkFlags"][-3] == '1': # HasName True
                Name_offset = shellLinkHeader["HeaderSize"]+sLinkTargetIDList_BlockSize+LinkInfo_BlockSize
                Name["length"] = struct.unpack('<H', binascii.unhexlify(data[Name_offset:Name_offset+2].encode('hex')))[0]
                Name_BlockSize = (Name["length"]*2)+2

                # NAME Parsing
                Name["StringData"] = data[Name_offset+2:Name_offset+Name["length"]*2+2].replace('\x00', '')

            if shellLinkHeader["sLinkFlags"][-4] == '1': # HasRelativePath True
                RelativePath_offset = shellLinkHeader["HeaderSize"]+sLinkTargetIDList_BlockSize+LinkInfo_BlockSize+Name_BlockSize
                RelativePath["length"] = struct.unpack('<H', binascii.unhexlify(data[RelativePath_offset:RelativePath_offset+2].encode('hex')))[0]
                RelativePath_BlockSize = (RelativePath["length"]*2)+2

                # RelativePath Parsing
                RelativePath["StringData"] = data[RelativePath_offset+2:RelativePath_offset+RelativePath["length"]*2+2].replace('\x00', '')

            if shellLinkHeader["sLinkFlags"][-5] == '1': # HasWorkingDir True
                WorkingDir_offset = shellLinkHeader["HeaderSize"]+sLinkTargetIDList_BlockSize+LinkInfo_BlockSize+Name_BlockSize+RelativePath_BlockSize
                WorkingDir["length"] = struct.unpack('<H', binascii.unhexlify(data[WorkingDir_offset:WorkingDir_offset+2].encode('hex')))[0]
                WorkingDir_BlockSize = (WorkingDir["length"]*2)+2

                # WorkingDir Parsing
                WorkingDir["StringData"] = data[WorkingDir_offset+2:WorkingDir_offset+WorkingDir["length"]*2+2].replace('\x00', '')

            if shellLinkHeader["sLinkFlags"][-6] == '1': # HasArguments True
                commandLine_offset = shellLinkHeader["HeaderSize"]+sLinkTargetIDList_BlockSize+LinkInfo_BlockSize+Name_BlockSize+RelativePath_BlockSize+WorkingDir_BlockSize
                commandLine["length"] = struct.unpack('<H', binascii.unhexlify(data[commandLine_offset:commandLine_offset+2].encode('hex')))[0]
                commandLine_BlockSize = (commandLine["length"]*2)+2

                # CommandLine Parsing
                commandLine["StringData"] = data[commandLine_offset+2:commandLine_offset+commandLine["length"]*2+2].replace('\x00', '')

            if shellLinkHeader["sLinkFlags"][-7] == '1': # HasIconLocation True
                iconLocation_offset = shellLinkHeader["HeaderSize"]+sLinkTargetIDList_BlockSize+LinkInfo_BlockSize+Name_BlockSize+RelativePath_BlockSize+WorkingDir_BlockSize+commandLine_BlockSize
                iconLocation["length"] = struct.unpack('<H', binascii.unhexlify(data[iconLocation_offset:iconLocation_offset+2].encode('hex')))[0]
                iconLocation_BlockSize = (iconLocation["length"]*2)+2

                # IconLocation Parsing
                iconLocation["StringData"] = data[iconLocation_offset+2:iconLocation_offset+iconLocation["length"]*2+2].replace('\x00', '')

            ExtraData_offset = shellLinkHeader["HeaderSize"]+sLinkTargetIDList_BlockSize+LinkInfo_BlockSize+Name_BlockSize+RelativePath_BlockSize+WorkingDir_BlockSize+commandLine_BlockSize+iconLocation_BlockSize

            TrackerData_offset = ExtraData_offset+data[ExtraData_offset:].find("\x03\x00\x00\xA0")-4 # -4 is signature offset

            # TrackerData Parsing
            TrackerData["BlockSize"] = struct.unpack('<l', binascii.unhexlify(data[TrackerData_offset:TrackerData_offset+4].encode('hex')))[0]
            TrackerData["Length"] = struct.unpack('<l', binascii.unhexlify(data[TrackerData_offset+8:TrackerData_offset+12].encode('hex')))[0]
            TrackerData["MachineID"] = data[TrackerData_offset+16:TrackerData_offset+32].replace('\x00', '')
            TrackerData["MacAddress"] = data[TrackerData_offset+58:TrackerData_offset+64].encode('hex')

            #ExtraData Parsing
            ExtraDataBlock_count, ExtraDataBlock_SID = SID_Extra(data[ExtraData_offset:])

            # result Output Call
            if output_type == 'display':
                auto_result_print(f_name, s_Name, sLinkTargetIDList, LinkInfo, Name, RelativePath, WorkingDir, commandLine, iconLocation, VolumeID, TrackerData, sLinkTargetIDList_CLSID, ExtraDataBlock_SID, 'display')
            elif output_type == 'csv':
                auto_result_print(f_name, s_Name, sLinkTargetIDList, LinkInfo, Name, RelativePath, WorkingDir, commandLine, iconLocation, VolumeID, TrackerData, sLinkTargetIDList_CLSID, ExtraDataBlock_SID, 'csv')
            elif output_type == 'sqlite':
                auto_result_print(f_name, s_Name, sLinkTargetIDList, LinkInfo, Name, RelativePath, WorkingDir, commandLine, iconLocation, VolumeID, TrackerData, sLinkTargetIDList_CLSID, ExtraDataBlock_SID, 'sqlite')

        # DestList Parsing
        elif s_Name == "DestList":
            if output_type == 'display':
                DestList_func(f_name, data, 'display')
            elif output_type == 'csv':
                DestList_func(f_name, data, 'csv')
            elif output_type == 'sqlite':
                DestList_func(f_name, data, 'sqlite')
    except:
        pass

# CustomDestinations Parser
def CustomParser(filename, output_type):
    fp = open(filename, 'rb')
    data = fp.read()
    fp.close()

    ExtraDataBlock_Path = ''

    lnk_offset = list()

    # shellLink structure start offset finding
    temp = 0
    while 1:
        temp = data.find("\x4C\x00\x00\x00\x01\x14\x02\x00\x00\x00\x00\x00\xC0\x00\x00\x00\x00\x00\x00\x46", temp)
        lnk_offset.append(temp)

        if temp == -1:
            lnk_offset.remove(-1)
            break

        temp += 20

    # Custom Lnk Parsing
    for offset in lnk_offset:
        print " [-] Lnk Structure Carving ..."
        sLinkTargetIDList_offset = 0
        sLinkTargetIDList_BlockSize = 0
        LinkInfo_offset = 0
        LinkInfo_BlockSize = 0
        Name_offset = 0
        Name_BlockSize = 0
        RelativePath_offset = 0
        RelativePath_BlockSize = 0
        WorkingDir_offset = 0
        WorkingDir_BlockSize = 0
        commandLine_offset = 0
        commandLine_BlockSize = 0
        iconLocation_offset = 0
        iconLocation_BlockSize = 0
        ExtraData_offset = 0
        TrackerData_offset = 0
        sLinkTargetIDList_count = 0
        sLinkTargetIDList_CLSID = list()
        ExtraDataBlock_count = 0
        ExtraDataBlock_SID = list()

        try:
            # ShellLinkHeader Parsing
            shellLinkHeader["HeaderSize"] = struct.unpack('<L', binascii.unhexlify(data[offset+0:offset+4].encode('hex')))[0]
            shellLinkHeader["FileAttributes"] = "{0:b}".format(struct.unpack("<HH", binascii.unhexlify(data[offset+24:offset+28].encode('hex')))[0])
            shellLinkHeader["CreationTimeL"] = struct.unpack("<l", binascii.unhexlify(data[offset+28:offset+32].encode('hex')))[0]
            shellLinkHeader["CreationTimeH"] = struct.unpack("<l", binascii.unhexlify(data[offset+32:offset+36].encode('hex')))[0]
            shellLinkHeader["ModifiedTimeL"] = struct.unpack("<l", binascii.unhexlify(data[offset+36:offset+40].encode('hex')))[0]
            shellLinkHeader["ModifiedTimeH"] = struct.unpack("<l", binascii.unhexlify(data[offset+40:offset+44].encode('hex')))[0]
            shellLinkHeader["AccessTimeL"] = struct.unpack("<l", binascii.unhexlify(data[offset+44:offset+48].encode('hex')))[0]
            shellLinkHeader["AccessTimeH"] = struct.unpack("<l", binascii.unhexlify(data[offset+48:offset+52].encode('hex')))[0]
            shellLinkHeader["FileSize"] = struct.unpack('<L', binascii.unhexlify(data[offset+52:offset+56].encode('hex')))[0]
            shellLinkHeader["ShowCommand"] = struct.unpack('<l', binascii.unhexlify(data[offset+60:offset+64].encode('hex')))[0]
            shellLinkHeader["sLinkFlags"] = "{0:b}".format(struct.unpack("<HH", binascii.unhexlify(data[offset+20:offset+24].encode('hex')))[0])

            # Link Flags Check
            if shellLinkHeader["sLinkFlags"][-1] == '1': # HasLinkTargetIDList True
                sLinkTargetIDList_offset = shellLinkHeader["HeaderSize"]+offset
                sLinkTargetIDList["LinkTargetIDListSize"] = struct.unpack('<H', binascii.unhexlify(data[sLinkTargetIDList_offset:sLinkTargetIDList_offset+2].encode('hex')))[0]
                sLinkTargetIDList_BlockSize = sLinkTargetIDList["LinkTargetIDListSize"]+2

                # sLinkTargetIDList Parsing
                sLinkTargetIDList_count, sLinkTargetIDList_CLSID = sLinkTargetIDList_func(data[sLinkTargetIDList_offset:sLinkTargetIDList_offset+sLinkTargetIDList["LinkTargetIDListSize"]+2])
                sLinkTargetIDList["TerminalID"] = data[sLinkTargetIDList_offset+sLinkTargetIDList["LinkTargetIDListSize"]-2:sLinkTargetIDList_offset+sLinkTargetIDList["LinkTargetIDListSize"]].encode('hex')

            if shellLinkHeader["sLinkFlags"][-2] == '1': # HasLinkInfo True
                LinkInfo_offset = shellLinkHeader["HeaderSize"]+sLinkTargetIDList_BlockSize+offset
                LinkInfo["LinkInfoSize"] = struct.unpack('<l', binascii.unhexlify(data[LinkInfo_offset:LinkInfo_offset+4].encode('hex')))[0]
                LinkInfo_BlockSize = LinkInfo["LinkInfoSize"]

                # LinkInfo Parsing
                LinkInfo["LinkInfoHeaderSize"] = struct.unpack('<l', binascii.unhexlify(data[LinkInfo_offset+4:LinkInfo_offset+8].encode('hex')))[0]
                LinkInfo["VolumeIDOffset"] = struct.unpack('<l', binascii.unhexlify(data[LinkInfo_offset+12:LinkInfo_offset+16].encode('hex')))[0]
                LinkInfo["LocalBasePathOffset"] = struct.unpack('<l', binascii.unhexlify(data[LinkInfo_offset+16:LinkInfo_offset+20].encode('hex')))[0]
                LinkInfo["LocalBasePath"] = data[LinkInfo_offset+LinkInfo["LocalBasePathOffset"]:LinkInfo_offset+LinkInfo["LinkInfoSize"]-2]

                # VolumeID Parsing
                VolumeID["VolumeSize"] = struct.unpack('<l', binascii.unhexlify(data[LinkInfo_offset+28:LinkInfo_offset+32].encode('hex')))[0]
                VolumeID["DriveType"] = struct.unpack('<l', binascii.unhexlify(data[LinkInfo_offset+32:LinkInfo_offset+36].encode('hex')))[0]
                VolumeID["DriveSerialNumber"] =struct.unpack('<L', binascii.unhexlify(data[LinkInfo_offset+36:LinkInfo_offset+40].encode('hex')))[0]

            if shellLinkHeader["sLinkFlags"][-3] == '1': # HasName True
                Name_offset = shellLinkHeader["HeaderSize"]+sLinkTargetIDList_BlockSize+LinkInfo_BlockSize+offset
                Name["length"] = struct.unpack('<H', binascii.unhexlify(data[Name_offset:Name_offset+2].encode('hex')))[0]
                Name_BlockSize = (Name["length"]*2)+2

                # NAME Parsing
                Name["StringData"] = data[Name_offset+2:Name_offset+Name["length"]*2+2].replace('\x00', '')

            if shellLinkHeader["sLinkFlags"][-4] == '1': # HasRelativePath True
                RelativePath_offset = shellLinkHeader["HeaderSize"]+sLinkTargetIDList_BlockSize+LinkInfo_BlockSize+Name_BlockSize+offset
                RelativePath["length"] = struct.unpack('<H', binascii.unhexlify(data[RelativePath_offset:RelativePath_offset+2].encode('hex')))[0]
                RelativePath_BlockSize = (RelativePath["length"]*2)+2

                # RelativePath Parsing
                RelativePath["StringData"] = data[RelativePath_offset+2:RelativePath_offset+RelativePath["length"]*2+2].replace('\x00', '')

            if shellLinkHeader["sLinkFlags"][-5] == '1': # HasWorkingDir True
                WorkingDir_offset = shellLinkHeader["HeaderSize"]+sLinkTargetIDList_BlockSize+LinkInfo_BlockSize+Name_BlockSize+RelativePath_BlockSize+offset
                WorkingDir["length"] = struct.unpack('<H', binascii.unhexlify(data[WorkingDir_offset:WorkingDir_offset+2].encode('hex')))[0]
                WorkingDir_BlockSize = (WorkingDir["length"]*2)+2

                # WorkingDir Parsing
                WorkingDir["StringData"] = data[WorkingDir_offset+2:WorkingDir_offset+WorkingDir["length"]*2+2].replace('\x00', '')

            if shellLinkHeader["sLinkFlags"][-6] == '1': # HasArguments True
                commandLine_offset = shellLinkHeader["HeaderSize"]+sLinkTargetIDList_BlockSize+LinkInfo_BlockSize+Name_BlockSize+RelativePath_BlockSize+WorkingDir_BlockSize+offset
                commandLine["length"] = struct.unpack('<H', binascii.unhexlify(data[commandLine_offset:commandLine_offset+2].encode('hex')))[0]
                commandLine_BlockSize = (commandLine["length"]*2)+2

                # CommandLine Parsing
                commandLine["StringData"] = data[commandLine_offset+2:commandLine_offset+commandLine["length"]*2+2].replace('\x00', '')

            if shellLinkHeader["sLinkFlags"][-7] == '1': # HasIconLocation True
                iconLocation_offset = shellLinkHeader["HeaderSize"]+sLinkTargetIDList_BlockSize+LinkInfo_BlockSize+Name_BlockSize+RelativePath_BlockSize+WorkingDir_BlockSize+commandLine_BlockSize+offset
                iconLocation["length"] = struct.unpack('<H', binascii.unhexlify(data[iconLocation_offset:iconLocation_offset+2].encode('hex')))[0]
                iconLocation_BlockSize = (iconLocation["length"]*2)+2

                # IconLocation Parsing
                iconLocation["StringData"] = data[iconLocation_offset+2:iconLocation_offset+iconLocation["length"]*2+2].replace('\x00', '')

            ExtraData_offset = shellLinkHeader["HeaderSize"]+sLinkTargetIDList_BlockSize+LinkInfo_BlockSize+Name_BlockSize+RelativePath_BlockSize+WorkingDir_BlockSize+commandLine_BlockSize+iconLocation_BlockSize+offset

            TrackerData_offset = ExtraData_offset+data[ExtraData_offset:].find("\x03\x00\x00\xA0")-4 # -4 is signature offset

            # TrackerData Parsing
            TrackerData["BlockSize"] = struct.unpack('<l', binascii.unhexlify(data[TrackerData_offset:TrackerData_offset+4].encode('hex')))[0]
            TrackerData["Length"] = struct.unpack('<l', binascii.unhexlify(data[TrackerData_offset+8:TrackerData_offset+12].encode('hex')))[0]
            TrackerData["MachineID"] = data[TrackerData_offset+16:TrackerData_offset+32].replace('\x00', '')
            TrackerData["MacAddress"] = data[TrackerData_offset+58:TrackerData_offset+64].encode('hex')

            #ExtraData Parsing
            ExtraDataBlock_count, ExtraDataBlock_Path = SID_Extra(data[ExtraData_offset:])

            # Stream Attributes Check
            attr_list = list()
            if len(shellLinkHeader["FileAttributes"]) < 15:
                for j in range(15-len(shellLinkHeader["FileAttributes"])):
                    shellLinkHeader["FileAttributes"] = '0' + shellLinkHeader["FileAttributes"]

            for i in range(len(shellLinkHeader["FileAttributes"])):
                if shellLinkHeader["FileAttributes"][i] == '1':
                    attr_list.append(File_attr[i])
            lnk_attr = '|'.join(attr_list)

            # converting Time
            conv_cTime = conv_time(shellLinkHeader["CreationTimeL"], shellLinkHeader["CreationTimeH"])
            conv_mTime = conv_time(shellLinkHeader["ModifiedTimeL"], shellLinkHeader["ModifiedTimeH"])
            conv_aTime = conv_time(shellLinkHeader["AccessTimeL"], shellLinkHeader["AccessTimeH"])

            # Drive Type Check
            d_type = DriveType[VolumeID["DriveType"]]


            # MAC Address Assembly
            MAC_Address = ''.join((TrackerData["MacAddress"][0:2], "-", TrackerData["MacAddress"][2:4], "-", TrackerData["MacAddress"][4:6], "-", TrackerData["MacAddress"][6:8], \
                                "-", TrackerData["MacAddress"][8:10], "-", TrackerData["MacAddress"][10:12]))


            # IDLIST, ExtraData char remove
            IDList_CLSID = ''.join(sLinkTargetIDList_CLSID)
            IDList_CLSID = IDList_CLSID.replace('\x00', '').replace('\r', '').replace('\n', '').replace(',', '')
            ExtraData_Path = ''.join(ExtraDataBlock_Path)
            ExtraData_Path = ExtraData_Path.replace('\x00', '').replace('\r', '').replace('\n', '').replace(',', '')

            # output call
            if output_type == 'display':
                custom_result_ouput(filename, shellLinkHeader, IDList_CLSID, VolumeID, conv_aTime, conv_cTime, conv_mTime, lnk_attr, attr_list, d_type, \
                                    str(offset), LinkInfo, commandLine, iconLocation, Name, WorkingDir, RelativePath, ExtraData_Path, TrackerData, MAC_Address, 'display')
            elif output_type == 'csv':
                custom_result_ouput(filename, shellLinkHeader, IDList_CLSID, VolumeID, conv_aTime, conv_cTime, conv_mTime, lnk_attr, attr_list, d_type, \
                                    str(offset), LinkInfo, commandLine, iconLocation,  Name, WorkingDir, RelativePath, ExtraData_Path, TrackerData, MAC_Address, 'csv')
            elif output_type == 'sqlite':
                custom_result_ouput(filename, shellLinkHeader, IDList_CLSID, VolumeID, conv_aTime, conv_cTime, conv_mTime, lnk_attr, attr_list, d_type, \
                                    str(offset), LinkInfo, commandLine, iconLocation, Name, WorkingDir, RelativePath, ExtraData_Path, TrackerData, MAC_Address, 'sqlite')
        except:
            pass

# CustomDestination Analysis Result Output Function
def custom_result_ouput(filename, shellLinkHeader, IDList_CSIDL, VolumeID, conv_aTime, conv_cTime, conv_mTime, lnk_attr, attr_list, d_type, \
                        offset, LinkInfo, commandLine, iconLocation, Name, WorkingDir, RelativePath, ExtraData_Path, TrackerData, MAC_Address, output_type):
    global resultFileName

    # commandLine, iconLocation, Name, Relative, WorkingDir char remove
    commandLine["StringData"] = commandLine["StringData"].replace('\x00', '').replace('\r', '').replace('\n', '').replace(',', ' ')
    iconLocation["StringData"] = iconLocation["StringData"].replace('\x00', '').replace('\r', '').replace('\n', '').replace(',', ' ')
    Name["StringData"] = Name["StringData"].replace('\x00', '').replace('\r', '').replace('\n', '').replace(',', ' ')
    RelativePath["StringData"] = RelativePath["StringData"].replace('\x00', '').replace('\r', '').replace('\n', '').replace(',', ' ')
    WorkingDir["StringData"] = WorkingDir["StringData"].replace('\x00', '').replace('\r', '').replace('\n', '').replace(',', ' ')

    # display output
    if output_type == 'display':
        print " JumpList File Name : \t\t\t", filename
        print " Offset(10) : \t\t\t\t", offset
        print " Lnk Attributes : \t\t\t", lnk_attr
        print " Lnk Created Time(Local) : \t\t", conv_cTime
        print " Lnk Modified Time(Local) : \t\t", conv_mTime
        print " Lnk Accessed Time(Local) : \t\t", conv_aTime
        print " Drive Type : \t\t\t\t", d_type
        print " Lnk Size : \t\t\t\t", shellLinkHeader["FileSize"]
        print " Volume Serial Number : \t\t", VolumeID["DriveSerialNumber"]
        print " Local Base Path : \t\t\t", LinkInfo["LocalBasePath"]
        print " Relative Path : \t\t\t", RelativePath["StringData"]
        print " Working Directory : \t\t", WorkingDir["StringData"]
        print " Target File Description : \t\t", Name["StringData"]
        print " Command Arguments : \t\t\t", commandLine["StringData"]
        print " Icon Location : \t\t\t", iconLocation["StringData"]
        print " NetBIOS Name : \t\t\t", TrackerData["MachineID"]
        print " MAC Address : \t\t\t\t", MAC_Address
        print " ItemIDList : \t\t\t\t", IDList_CSIDL
        print " ExtraData : \t\t\t\t", ExtraData_Path
        print "\n"

    # csv output
    elif output_type == 'csv':
        fp = open(resultFileName+"_custom.csv", 'a')
        fp.write(filename+","+offset+","+lnk_attr+","+conv_cTime+","+conv_mTime+","+conv_aTime+","+d_type+","+str(shellLinkHeader["FileSize"])+","+str(VolumeID["DriveSerialNumber"])+\
                 ","+LinkInfo["LocalBasePath"]+","+RelativePath["StringData"]+","+WorkingDir["StringData"]+","+Name["StringData"]+","+commandLine["StringData"]+","+iconLocation["StringData"]+","+\
                 TrackerData["MachineID"]+","+MAC_Address+","+IDList_CSIDL+","+ExtraData_Path+"\n")
        fp.close()

    # sqlite output
    elif output_type == 'sqlite':
        # SQLite DB stream Table DATA INSERT
        try:
            con = sqlite3.connect(resultFileName+"_custom.db")
            cur = con.cursor()
            query = "INSERT INTO customlnk( \
            JumpListFileName, \
            Offset, \
            LnkAttributes, \
            CreatedTime, \
            ModifiedTime, \
            AccessedTime, \
            DriveType, \
            LnkSize, \
            VolumeSerialNumber, \
            LocalBasePath, \
            RelativePath, \
            WorkingDirectory, \
            TargetFileDescription, \
            CommandArguments, \
            IconLocation, \
            NetBIOSName, \
            MACAddress, \
            ItemIDList, \
            ExtraData) \
            VALUES(\"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\")"%(\
                filename, str(offset), lnk_attr, conv_cTime, conv_mTime, conv_aTime, d_type, str(shellLinkHeader["FileSize"]), str(VolumeID["DriveSerialNumber"]), \
                LinkInfo["LocalBasePath"], RelativePath["StringData"], WorkingDir["StringData"], Name["StringData"], commandLine["StringData"], iconLocation["StringData"], \
                TrackerData["MachineID"], MAC_Address, IDList_CSIDL.split("\"")[0], ExtraData_Path.split("\"")[0])

            cur.execute(query)
            con.commit()
        except:
            pass

# CustomDestination Result file Creation Function
def custom_file_creation(filename, flag):
    '''
    @filename data stored filename
    @flag file type [csv|sqlite]
    '''
    if flag == 'csv':
        f = open(filename+'_custom.csv', 'w')
        f.write("JumpList File Name,offset,Lnk Attributes,Custom Lnk Created Time(Local),Custom Lnk Modified Time(Local),Custom Lnk Accessed Time(Local),\
        Drive Type,Lnk Size,Volume Serial Number,Local Base Path, Relative Path,Working Directory,Target File Description,Command Arguments,IconLocation,NetBIOS Name, MAC Address,ItemIDList,ExtraData\n")
        f.close()

    elif flag == 'sqlite':
        # SQLite DB stream Table Create
        con = sqlite3.connect(filename+"_custom.db")
        cur = con.cursor()
        query = 'CREATE TABLE customlnk( \
        JumpListFileName TEXT, \
        Offset TEXT, \
        LnkAttributes TEXT, \
        CreatedTime TEXT, \
        ModifiedTime TEXT, \
        AccessedTime TEXT, \
        DriveType TEXT, \
        LnkSize TEXT, \
        VolumeSerialNumber TEXT, \
        LocalBasePath TEXT, \
        RelativePath TEXT, \
        WorkingDirectory TEXT, \
        TargetFileDescription TEXT, \
        CommandArguments TEXT, \
        IconLocation TEXT, \
        NetBIOSName TEXT, \
        MACAddress TEXT, \
        ItemIDList TEXT, \
        ExtraData TEXT);'

        try:
            cur.execute(query)
            con.commit()
        except:
            pass

# AutomaticDestination Analysis Result file Creation Function
def auto_file_creation(filename, flag, flag2):
    '''
    @filename data stored filename
    @flag file type [csv|sqlite]
    @flag2 data type [stream|destlist]
    '''

    if flag == "csv":
        if flag2 == 'stream':
            # Stream store
            fp = open(filename+"_stream.csv", 'w')
            fp.write("JumpList File Name,Stream Name,Stream Attributes,Stream Created Time(Local),Stream Modified Time(Local),Stream Accessed Time(Local),Drive Type,Stream Size,Volume \
            Serial Number,Local Base Path,Relative Path,Working Directory,File Description,Command Arguments,IconLocation,NetBIOS Name,MAC Address,ItemIDList,ExtraData\n")
            fp.close()

        elif flag2 == 'destlist':
            # DestList store
            fp = open(filename+"_destlist.csv", 'w')
            fp.write("FileName,StreamName,CheckSum,NewVolumeID,ObjectID_1,ObjectID_2,BirtuVolumeID,MachineID,FileAccessCounter,LastRecordTime,String Data\n")
            fp.close()

    elif flag == "sqlite":
       # SQLite DB stream Table Create
       con = sqlite3.connect(filename+"_automatic.db")
       cur = con.cursor()
       if flag2 == 'stream':
           query = 'CREATE TABLE stream( \
           JumpListFileName TEXT, \
           StreamName TEXT, \
           StreamAttributes TEXT, \
           StreamCreatedTime TEXT, \
           StreamModifiedTime TEXT, \
           StreamAccessedTime TEXT, \
           DriveType TEXT, \
           StreamSize TEXT, \
           VolumeSerialNumber TEXT, \
           LocalBasePath TEXT, \
           RelativePath TEXT, \
           WorkingDirectory TEXT, \
           FileDescription TEXT, \
           CommandArguments TEXT, \
           IconLocation TEXT, \
           NetBIOSName TEXT, \
           MACAddress TEXT, \
           ItemIDList TEXT, \
           ExtraData TEXT);'

           try:
               cur.execute(query)
               con.commit()
           except:
               pass

       elif flag2 == 'destlist':
           query = 'CREATE TABLE destlist( \
           JumpListFileName TEXT, \
           CheckSum TEXT, \
           NewVolumeID TEXT, \
           ObjectID_1 TEXT, \
           BirthVolumeID TEXT, \
           ObjectID_2 TEXT, \
           MachineID TEXT, \
           StreamName TEXT, \
           FileAccessCount TEXT, \
           LastRecordTime TEXT, \
           StringData TEXT);'

           try:
               cur.execute(query)
               con.commit()
           except:
               pass

# create File list in Directory
def dir_loof(dir):
    '''
    @dir -d value(directory)
    '''
    print "[+] Searching extension '.automaticDestinations-ms' and '.customDestinations-ms' ..."
    auto_fileList = list()
    custom_fileList = list()
    appName_list = list()
    for (path, dir, files) in os.walk(dir):
        for filename in files:
            appID = os.path.splitext(filename)[0]
            ext = os.path.splitext(filename)[-1]

            try:
                appName = AppIDList[appID]
            except:
                appName = "Not Found!"

            if ext == '.automaticDestinations-ms':
                print " [-] Find it %s/%s \t-> %s" % (path, filename, appName)
                auto_fileList.append("%s/%s" % (path, filename))

            if ext == '.customDestinations-ms':
                print " [-] Find it %s/%s \t-> %s" % (path, filename, appName)
                custom_fileList.append("%s/%s" % (path, filename))

    return auto_fileList, custom_fileList

# OLE File Parse
def OLEParser(filename, output_type):
    '''
    @filename OLE Filename
    @output_type output type [display|csv|sqlite]
    '''

    # Open OLE File
    ole = OleFileIO_PL.OleFileIO(filename)

    # Stream name sort
    stream_name = list()
    for name in ole.listdir():
        stream_name.append(''.join(name))
    stream_name.remove("DestList")
    stream_name.append("DestList")

    print " [-] JumpList Stream Extract ..."
    # stream data read
    for s_name in stream_name:
        streamOpen = ole.openstream(s_name)
        streamData = streamOpen.read()

        # Call LnkParser function
        LnkParser(filename, s_name, streamData, output_type)

if __name__ == '__main__':
    # intro printing
    print "\nWindows JumpList Forensics Analysis Tool v0.1 Made by MaJ3stY@Plainbit(http://plainbit.co.kr)"
    usage = """%prog [-f|-d] <Filename or Directory> [-c|-s] <Result Filename>
    ex1) %prog -f 1b4dd67f29cb1962.automaticDestinations-ms
    ex2) %prog -d ./ -c csv_dir_result
    ex2) %prog -d ./ -s sqlite_dir_result

    contact : junhyeong.lee@plainbit.co.kr"""

    parser = OptionParser(usage=usage)
    parser.add_option("-f", "--file", dest="filename", action="store_true", help="Option to be used for the analysis of a single file", default=False)
    parser.add_option("-d", "--dir", dest="directory", action="store_true", help="When analyzing more than one file, the option to specify the directory", default=False)
    parser.add_option("-c", "--csv", dest="csv", action="store_true", help="The output in the CSV when you save the file", default=False)
    parser.add_option("-s", "--sqlite", dest="sqlite", action="store_true", help="The output in SQLite when you save the file", default=False)

    (options, args) = parser.parse_args()
    if len(args) == 0 or len(args) < 1:
        parser.print_help()
        sys.exit(2)
    else:
        print "[+] Analysis Starting!\n"
        # -f
        if options.filename == True and options.csv == False and options.sqlite == False:

            if args[0].find('.automaticDestinations-ms') != -1:
                print "[+] AutomaticDestination Analysis ..."
                OLEParser(args[0], 'display')

            if args[0].find('.customDestinations-ms') != -1:
                print "[+] CustomDestination Analysis ..."
                CustomParser(args[0], 'display')

        # -d
        if options.directory == True and options.csv == False and options.sqlite == False:
            auto_filenames, custom_filenames = dir_loof(args[0]) # filename list create

            print "[+] AutomaticDestination Analysis ..."
            for f_name in auto_filenames: # OLEParser call loof
                OLEParser(f_name, 'display')

            print "[+] CustomDestination Analysis ..."
            for f_name in custom_filenames: # CustomParser call loof
                CustomParser(f_name, 'display')

        # -f <filename> -c <csv filename>
        if options.filename == True and options.csv == True:
            resultFileName = args[1]

            if args[0].find('.automaticDestinations-ms') != -1:
                # Stream stored CSV file creation
                auto_file_creation(resultFileName, 'csv', 'stream')
                print "[+] Creation Filename is '", resultFileName+"_stream.csv'"

                # DestList stored CSV file creation
                auto_file_creation(resultFileName, 'csv', 'destlist')
                print "[+] Creation Filename is '", resultFileName+"_destlist.csv'"

                # Call OLE Analysis function
                print "[+] AutomaticDestination Analysis ..."
                OLEParser(args[0], 'csv')

            if args[0].find('.customDestinations-ms') != -1:
                custom_file_creation(resultFileName, 'csv')
                print "[+] Creation Filename is '", resultFileName+"_custom.csv'"
                print "[+] CustomDestination Analysis ..."
                CustomParser(args[0], 'csv')

        # -f and <filename> -s <sqlite filename>
        if options.filename == True and options.sqlite == True:
            resultFileName = args[1]

            if args[0].find('.automaticDestinations-ms') != -1:
                # sqlite DB file creation
                auto_file_creation(resultFileName, 'sqlite', 'stream')
                auto_file_creation(resultFileName, 'sqlite', 'destlist')
                print "[+] Creation Filename is '", resultFileName+"_automatic.db'"

                # Call OLE analysis function
                print "[+] AutomaticDestination Analysis ..."
                OLEParser(args[0], 'sqlite')

            if args[0].find('.customDestinations-ms') != -1:
                custom_file_creation(resultFileName, 'sqlite')
                print "[+] Creation Filename is '", resultFileName+"_custom.db'"
                print "[+] CustomDestination Analysis ..."
                CustomParser(args[0], 'sqlite')

        # -d <directory path> -c <csv filename>
        if options.directory == True and options.csv == True:
            resultFileName = args[1]

            auto_filenames, custom_filenames = dir_loof(args[0]) # filename list create


            if len(auto_filenames) > 0:
                # Stream stored CSV file creation
                auto_file_creation(resultFileName, 'csv', 'stream')
                print "[+] Creation Filename is '", resultFileName+"_stream.csv'"

                # DestList stored CSV file creation
                auto_file_creation(resultFileName, 'csv', 'destlist')
                print "[+] Creation Filename is '", resultFileName+"_destlist.csv'"

                print "[+] AutomaticDestination Analysis ..."
                for f_name in auto_filenames: # OLEParser call loof
                    # Call OLE Analysis function
                    OLEParser(f_name, 'csv')


            if len(custom_filenames) > 0:
                custom_file_creation(resultFileName, 'csv')
                print "[+] Creation Filename is '", resultFileName+"_custom.csv'"
                print "[+] CustomDestination Analysis ..."
                for f_name in custom_filenames: # CustomParser call loof
                    CustomParser(f_name, 'csv')

        # -d <directory path> -s <sqlite filename>
        if options.directory == True and options.sqlite == True:
            resultFileName = args[1]

            auto_filenames, custom_filenames = dir_loof(args[0]) # filename list create

            if len(auto_filenames) > 0:
                # sqlite DB file creation
                auto_file_creation(resultFileName, 'sqlite', 'stream')
                auto_file_creation(resultFileName, 'sqlite', 'destlist')
                print "[+] Creation Filename is '", resultFileName+"_automatic.db'"

                print "[+] AutomaticDestination Analysis ..."
                for f_name in auto_filenames: # OLEParser call loof
                    # Call OLE Analysis function
                    OLEParser(f_name, 'sqlite')

            if len(custom_filenames) > 0:
                custom_file_creation(resultFileName, 'sqlite')
                print "[+] Creation Filename is '", resultFileName+"_custom.db'"
                print "[+] CustomDestination Analysis ..."
                for f_name in custom_filenames: # CustomParser call loof
                    # Call CustomDestination Analysis
                    CustomParser(f_name, 'sqlite')

    print "\n[+] Analysis End!"